/*
 *  Copyright (c) 2004 Apple Computer, Inc. All Rights Reserved.
 * 
 *  @APPLE_LICENSE_HEADER_START@
 *  
 *  This file contains Original Code and/or Modifications of Original Code
 *  as defined in and that are subject to the Apple Public Source License
 *  Version 2.0 (the 'License'). You may not use this file except in
 *  compliance with the License. Please obtain a copy of the License at
 *  http://www.opensource.apple.com/apsl/ and read it before using this
 *  file.
 *  
 *  The Original Code and all software distributed under the License are
 *  distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 *  EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 *  INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 *  Please see the License for the specific language governing rights and
 *  limitations under the License.
 *  
 *  @APPLE_LICENSE_HEADER_END@
 */

/*
 *  Relation.cpp
 *  TokendMuscle
 */

#include "Relation.h"

namespace Tokend
{

// @@@ need to distinguish between records that exist at db open time, and
// those that are being added and must be written back to the card

#pragma mark ---------------- Relation methods --------------

Relation::~Relation()
{
	delete mMetaRecord;
}

void Relation::insertRecord(const RefPointer<Record> &record)
{
	push_back(record);
}

bool Relation::matchesId(RelationId inRelationId) const
{
	RelationId anId = mMetaRecord->relationId();
	if (inRelationId == CSSM_DL_DB_RECORD_ANY) // All non schema tables.
		return !(CSSM_DB_RECORDTYPE_SCHEMA_START <= anId
			&& anId < CSSM_DB_RECORDTYPE_SCHEMA_END);

	if (inRelationId == CSSM_DL_DB_RECORD_ALL_KEYS) // All key tables.
		return (anId == CSSM_DL_DB_RECORD_PUBLIC_KEY
				|| anId == CSSM_DL_DB_RECORD_PRIVATE_KEY
				|| anId == CSSM_DL_DB_RECORD_SYMMETRIC_KEY);

	return inRelationId == anId; // Only if exact match.
}


} // end namespace Tokend

/* arch-tag: E9350280-DF80-11D8-B395-000A95C4302E */
